@@ -1,17 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2016-2021 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import subprocess
 import re
 import os
@@ -184,7 +173,7 @@ class WindowsTests(BasePlatformTests):
             # to the right reason).
             return
         self.build()
-    
+
     @skipIf(is_cygwin(), 'Test only applicable to Windows')
     def test_genvslite(self):
         # The test framework itself might be forcing a specific, non-ninja backend across a set of tests, which
@@ -195,7 +184,7 @@ class WindowsTests(BasePlatformTests):
         if self.backend is not Backend.ninja:
             raise SkipTest('Test only applies when using the Ninja backend')
 
-        testdir = os.path.join(self.unit_test_dir, '115 genvslite')
+        testdir = os.path.join(self.unit_test_dir, '117 genvslite')
 
         env = get_fake_env(testdir, self.builddir, self.prefix)
         cc = detect_c_compiler(env, MachineChoice.HOST)
@@ -225,7 +214,7 @@ class WindowsTests(BasePlatformTests):
         # Wrap the following bulk of setup and msbuild invocation testing in a try-finally because any exception,
         # failure, or success must always clean up any of the suffixed build dir folders that may have been generated.
         try:
-            # Since this 
+            # Since this
             self.init(testdir, extra_args=['--genvslite', 'vs2022'])
             # We need to bear in mind that the BasePlatformTests framework creates and cleans up its own temporary
             # build directory.  However, 'genvslite' creates a set of suffixed build directories which we'll have
@@ -462,6 +451,9 @@ class WindowsTests(BasePlatformTests):
 
     @unittest.skipIf(is_cygwin(), "Needs visual studio")
     def test_vsenv_option(self):
+        if mesonbuild.environment.detect_msys2_arch():
+            # https://github.com/msys2-contrib/cpython-mingw/issues/141
+            raise SkipTest('mingw python fails with /bin being removed from PATH')
         if self.backend is not Backend.ninja:
             raise SkipTest('Only ninja backend is valid for test')
         env = os.environ.copy()
